{% extends "!page.html" %}

{%- block htmltitle %}
<title>{{ title|striptags|e }}{%- for parent in parents %} – {{ parent.title }}{%- endfor %}{{ titlesuffix }}</title>
{%- endblock %}

{% block body %}
<div>
    <h1>Page not found</h1>
    <p>We could not find the page you requested in the icalendar documentation.</p>
    <p>You can <a href="search.html">search</a> the icalendar documentation.</p>
    <p>You can use the version switcher to go to other versions of icalendar.</p>
    <h2>Why is this happening?</h2>
    <p>
        The icalendar documentation team has restructured documentation to redirect visitors to the most current and accurate information, archive previous versions, and identify outdated pages.
        If you would like to report an issue or request a redirect be set up, you can <a href="https://github.com/collective/icalendar/issues/new?template=bug_report.md" rel="external">create a new issue in the GitHub issue tracker</a>.
    </p>
</div>

{% endblock %}
